"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
exports.preprocessReadme = void 0;
var compiler_1 = require("svelte/compiler");
var markdown_it_1 = __importDefault(require("markdown-it"));
var markdown_it_anchor_1 = __importDefault(require("markdown-it-anchor"));
var prettier_1 = __importDefault(require("prettier"));
var prismjs_1 = __importDefault(require("prismjs"));
require("prismjs/components/prism-bash");
require("prism-svelte");
var is_relative_url_1 = __importDefault(require("is-relative-url"));
var url_1 = require("url");
var aliases = {
    sh: "bash",
    js: "javascript"
};
var md;
function preprocessReadme(opts) {
    var prefixUrl = opts.prefixUrl || opts.homepage + "/tree/master/";
    if (!md) {
        md = new markdown_it_1["default"]({
            html: true,
            linkify: true,
            typographer: true,
            highlight: function (source, lang) {
                if (lang === "svelte") {
                    var regex = new RegExp('"' + opts.name + '"', "g");
                    var modifiedSource = encodeURI(source.replace(regex, '"' + opts.svelte + '"'));
                    var formattedCode = prettier_1["default"].format(source, {
                        parser: "svelte",
                        // @ts-ignore
                        svelteBracketNewLine: true
                    });
                    var svelteCode = prismjs_1["default"].highlight(formattedCode, prismjs_1["default"].languages.svelte, "svelte");
                    return "<pre class=\"language-" + lang + "\" data-svelte=\"" + modifiedSource + "\">{@html `" + svelteCode + "`}</pre>";
                }
                try {
                    var alias_lang = aliases[lang] || lang;
                    return "<pre class=\"language-" + alias_lang + "\">{@html `" + prismjs_1["default"].highlight(source, prismjs_1["default"].languages[alias_lang], alias_lang) + "`}</pre>";
                }
                catch (e) {
                    console.error("Could not highlight language \"" + lang + "\".");
                    return "";
                }
            }
        });
        md.use(markdown_it_anchor_1["default"]);
    }
    return {
        // @ts-ignore
        markup: function (_a) {
            var content = _a.content, filename = _a.filename;
            if (/node_modules/.test(filename) || !filename.endsWith(".md"))
                return null;
            if (opts.repoUrl) {
                content = content.replace("<!-- REPO_URL -->", "[GitHub repo](" + opts.repoUrl + ")");
            }
            content = content.replace("<!-- TOC -->", "\n## Table of Contents\n      ");
            var script_content = "";
            var style_content = "";
            var result = md.render(content);
            var cursor = 0;
            var ast = compiler_1.parse(result);
            var headings = [];
            var prev = undefined;
            compiler_1.walk(ast, {
                enter: function (node, parent) {
                    if (node.type === "Attribute" && node.name === "href") {
                        var value = node.value[0];
                        if (value && !value.raw.startsWith("#") && is_relative_url_1["default"](value.raw)) {
                            var relative_path = new url_1.URL(value.raw, prefixUrl).href;
                            result = result.replace(value.raw, relative_path);
                            cursor += relative_path.length - value.raw.length;
                        }
                    }
                    if (node.type === "Style") {
                        style_content += result.slice(node.content.start, node.content.end);
                        var replace_style = result.slice(node.start + cursor, node.end + cursor);
                        result = result.replace(replace_style, "");
                        cursor -= replace_style.length;
                    }
                    if (node.type === "Element" && node.name === "h2") {
                        // @ts-ignore
                        var id = node.attributes.find(function (attr) { return attr.name === "id"; }).value[0].raw;
                        if (id === "table-of-contents")
                            return;
                        var text = node.children[0].raw;
                        if (text !== undefined) {
                            if (prev === "h3") {
                                headings.push("</ul><li><a href=\"#" + id + "\">" + text + "</a></li>");
                            }
                            else {
                                headings.push("<li><a href=\"#" + id + "\">" + text + "</a></li>");
                            }
                            prev = "h2";
                        }
                    }
                    if (node.type === "Element" && node.name === "h3") {
                        // @ts-ignore
                        var id = node.attributes.find(function (attr) { return attr.name === "id"; }).value[0].raw;
                        var text = node.children[0].raw;
                        if (text !== undefined) {
                            if (prev === "h2") {
                                headings.push("<ul><li><a href=\"#" + id + "\">" + text + "</a></li>");
                            }
                            else {
                                headings.push("<li><a href=\"#" + id + "\">" + text + "</a></li>");
                            }
                            prev = "h3";
                        }
                    }
                    if (node.type === "Attribute" && node.name === "data-svelte") {
                        var raw_value = node.value[0].raw;
                        var value_1 = decodeURI(raw_value);
                        var value_ast = compiler_1.parse(value_1);
                        var markup = "<div class=\"code-fence\">" + value_1.slice(value_ast.html.start, value_ast.html.end) + "</div>";
                        var replace = result.slice(parent.start + cursor, parent.end + cursor);
                        result = result.replace(replace, markup + replace.replace(raw_value, ""));
                        cursor += markup.length - raw_value.length;
                        compiler_1.walk(value_ast, {
                            enter: function (node) {
                                if (node.type === "Script") {
                                    script_content += value_1.slice(node.content.start, node.content.end);
                                }
                            }
                        });
                    }
                }
            });
            if (prev === "h3") {
                headings.push("</ul>");
            }
            result = result.replace("<h2 id=\"table-of-contents\">Table of Contents</h2>", "<p><strong>Table of Contents</strong></p><ul>" + headings.join("\n") + "</ul>");
            return {
                code: "<script>" + script_content + "</script>\n               <style>" + style_content + "</style>\n               <main class=\"markdown-body\">" + result + "</main>"
            };
        }
    };
}
exports.preprocessReadme = preprocessReadme;
